package cz.jikos.ales.mff.prgj.cfinder;

import java.io.File;
import java.text.NumberFormat;
import java.util.LinkedList;

/**
 * Abstract class containing functions/fields common to
 * all the class-finding jobs.
 * 
 * @author alesak@jikos.cz
 */
public abstract class ClassFinder {
	protected Storage storage;
	protected String home;
	protected boolean hasToIndex;
	protected File storageHome;
	protected String storageDir = null;
	protected LinkedList<File> dirsTodo = new LinkedList<File>();
	protected boolean hasToQuery;
	protected String queryClassName;
	protected String queryFileName;
	protected String queryPackageName;
	protected String queryMethodName;

	/**
	 * from specified arguments determine, if GUI should be run
	 * 
	 * @param args
	 * @return
	 */
	protected static boolean isGui (String [] args) {
		for (String s : args) {
			if (s.equals("--index")) return false;
			else if (s.equals("--query")) return false;
		}
		return true;
	}
	
	/**
	 * Parses arguments and sets fields respectively.
	 * 
	 * @param args
	 */
	protected void parseArgs(String[] args) {

		try {
			for (int i = 0; i < args.length; i++) {
				String s = args[i];
				if (s.equals("--storage")) {
					storageDir = args[++i];
				} else if (s.equals("--query")) {
					hasToQuery = true;
				} else if (s.equals("--index")) {
					hasToIndex = true;
				} else if (s.equals("--class")) {
					queryClassName = args[++i];
				} else if (s.equals("--package")) {
					queryPackageName = args[++i];
				} else if (s.equals("--method")) {
					queryMethodName = args[++i];
				} else if (s.equals("--file")) {
					queryFileName = args[++i];
				} else if (s.equals("--dir")) {
					dirsTodo.add(new File (args [++i]));
				} else {
					System.err.println("Unknown argument " + args);
				}
			}
		} catch (IndexOutOfBoundsException e) {
			throw new RuntimeException("Bad arguments");
		}
	}

	protected void printUsage() {
		Runtime runtime = Runtime.getRuntime();
		NumberFormat format = NumberFormat.getInstance();

		StringBuilder sb = new StringBuilder();
		long maxMemory = runtime.maxMemory();
		long allocatedMemory = runtime.totalMemory();
		long freeMemory = runtime.freeMemory();

		sb.append("free memory: " + format.format(freeMemory / 1024) + "<br/>");
		sb.append("allocated memory: " + format.format(allocatedMemory / 1024)
				+ "<br/>");
		sb.append("max memory: " + format.format(maxMemory / 1024) + "<br/>");
		sb.append("total free memory: "
				+ format.format((freeMemory + (maxMemory - allocatedMemory)) / 1024)
				+ "<br/>");
		System.out.println(sb.toString());
	}

	/**
	 * Initializes paths.
	 */
	public void initializePaths()
	{
		do {
			home = System.getenv("HOME");
			if (home != null)
				break;
			String[] wenv = { "HOMEDRIVE", "HOMEPATH", "USERPROFILE" };
			home = "";
			for (String s : wenv)
				if (System.getenv().containsKey(s))
					home += System.getenv(s) + File.separator;
			if (home.length() > 0) {
				home = home.substring(0, home.length() - 1);
				break;
			}
			throw new RuntimeException("Cannot find home directory");
		} while (false);

		if (storageDir == null) storageDir = home + File.separator + ".classfinder";		
	}
	
	/**
	 * Initializes storage.
	 */
	public void initializeStorage() {
		storageHome = new File(storageDir);

		if (!storageHome.exists()) {
			if (!storageHome.mkdirs())
				throw new RuntimeException("Cannot create directory "
						+ storageHome);
		}

		storage = StorageFactory.storageFactoryMethod(storageHome);
	}
}
