package cz.jikos.ales.mff.prgj.cfinder;

import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;

import cz.jikos.ales.mff.prgj.simpleser.SimSer;
@SimSer.Serialize
/**
 * Object encapsulating all the info about the class.
 * 
 * @author alesak@jikos.cz
 */
public class ClsInfo {
	@SimSer.Serialize
	/** class name */
	public String name;
	@SimSer.Serialize
	/** package info structure */
	public PkgInfo pkg;
	@SimSer.Serialize
	/** classfile name */
	public String fileName;
	@SimSer.Serialize
	/** date of last modify */
	public long date;
	
	@SimSer.Serialize
	public List<Method> methods = new LinkedList<ClsInfo.Method>();
	@SimSer.Serialize
	public List<Field> fields = new LinkedList<ClsInfo.Field>();
	
	@SimSer.Serialize
	/**
	 * Method info struct.
	 */
	public static class Method {
		@SimSer.Serialize
		public String name;
		public String toString() {
			return (super.toString() + ":name=" + name + ";");
		}
	}
	
	@SimSer.Serialize
	/**
	 * Field info struct.
	 */
	public static class Field {
		@SimSer.Serialize
		public String name;
		public String toString() {
			return (super.toString() + ":name=" + name + ";");
		}
	}
	/**
	 * Compares if string s matches query string query
	 * @param s
	 * @param query
	 * @param caseInSensitive
	 * @return
	 */
	protected static boolean matches (String s, String query, boolean caseInSensitive) {
		if (query == null) return true;
		if (s == null) return false;
		if (caseInSensitive) {
			s = s.toUpperCase();
			query = query.toUpperCase();
		}
		if (query.endsWith("*")) {
			query = query.substring(0, query.length() - 1);
			return (s.startsWith(query));
		}
		return s.equals(query);
	}
	
	/**
	 * Determines, if this objects matches specified query
	 * 
	 * @param query
	 * @param caseInSensitive
	 * @return
	 */
	public boolean matches (ClsInfo query, boolean caseInSensitive) {
		do {
			if (! matches (name, query.name, caseInSensitive)) break;

			if (! matches (fileName, query.fileName, caseInSensitive)) break;
			
			if (pkg != null && query.pkg != null) {
				if (! matches (pkg.name, query.pkg.name, caseInSensitive)) break;
			}
			boolean found = true;

			for (ClsInfo.Method m : query.methods) {
				found = false;
				for (ClsInfo.Method mm : methods) {
					if (matches (mm.name, m.name, caseInSensitive)) {
						found = true;
						break;
					}
				}
				if (! found) break;
			}
			if (! found) break;

			return true;
		} while (false);
		
		return false;
	}
	
	public String toString ()
	{
		StringBuilder sb = new StringBuilder();
		sb.append(super.toString());
		sb.append(":").append("className=").append(name).append(",pkg=").append((pkg!=null)?pkg.name:"<null>");
		sb.append(",fields={");
		for (Field f : fields) {
			sb.append(f.toString());
		}
		sb.append("},methods={");
		for (Method m : methods)
			sb.append(m.toString());
		sb.append("}");
		
		return sb.toString();
	}
	
	public ClsInfo () {
	}
}
