package cz.jikos.ales.mff.prgj.cfinder;

/**
 * Interface of storage.
 * 
 * @author alesak@jikos.cz
 */
public interface Storage {
	/**
	 * Interface used as callback when searching
	 * for classes.
	 * 
	 * @author alesak@jikos.cz
	 */
	public interface Callback {
		public boolean call(ClsInfo object);
	}
	
	/**
	 * Exception encapsulating storage errors.
	 * 
	 * @author alesak@jikos.cz
	 */
	public static class StorageException extends RuntimeException {
		public StorageException (String msg) {
			super (msg);
		}
		private static final long serialVersionUID = -538343473094145819L;
	}
	
	/**
	 * Runs query on storage. The method of search is up to 
	 * the implementation.
	 * 
	 * @param query @ref ClsInfo defining the query. Non-empty fields
	 *   are serving as conditions
	 * @param limit maximal count of object to retrieve
	 * @param callback if not null, is called for every found object
	 * @return array of all the objects found
	 */
	public ClsInfo[] findClsInfo (ClsInfo query, int limit, Callback callback);
	
	/**
	 * Stores @ref ClsInfo into storage. It's up to implementation,
	 * what further indexing it does.
	 * 
	 * @param path path to class file
	 * @param clsInfo ClsInfo to store
	 */
	public void storeClsInfo (String path, ClsInfo clsInfo);
	
	/**
	 * @return true, if the storage is empty
	 */
	public boolean isEmpty ();
	
	/**
	 * Explicitly closes storage resources.
	 */
	public void close();
	
	/**
	 * Moves this storage's files from one directory
	 * to another.
	 * 
	 * @param source from-dir
	 * @param dest to-dir
	 */
	public void move(String source, String dest);
}
