package cz.jikos.ales.mff.prgj.cfinder;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.NumberFormat;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;

public class Trie {
	static int count = 0;
	
	private Map<Character,Trie> childs = new TreeMap<Character,Trie>();
	private LinkedList<Object> objs = null;
	
	public Trie () { count ++; }
	
	private void putObject (Object o) {
		if (objs == null) objs = new LinkedList<Object>();
		objs.push(o);
	}
	
	
	public void putObject (String str, Object o) {
		Trie t;
		int i;
		
		t = this;
		
		for (i = 0; i < str.length(); i ++) {
			char ch = str.charAt(i);
			if (t.childs.containsKey(ch)) {
				t = t.childs.get(ch);
			} else {
				break;
			}
		}
		
		for (; i < str.length(); i ++) {
			char ch = str.charAt(i);
			Trie nt = new Trie();
			t.childs.put(ch, nt);
			t = nt;
		}
		
		t.putObject(o);
	}
	
	public Object[] getObjects (String str) {
		Trie t;
		int i;
		
		t = this;
		for (i = 0; i < str.length(); i ++) {
			char ch = str.charAt(i);
			if (! t.childs.containsKey(ch)) return null;
			t = t.childs.get(ch);
		}
		
		if (t.objs == null) return null;
		else return (t.objs.toArray());
		
	}
	
	public static void main(String[] args) {
		File f = new File ("/tmp/classes");
		Trie trie = new Trie();
		
		try {
			String s;
			
			FileInputStream fis = new FileInputStream(f);
			
			BufferedReader br = new BufferedReader(new InputStreamReader(fis));
			
			while ((s = br.readLine()) != null) {
				trie.putObject(s, s);
			}
			
			String ss[] = {"/usr/share/gettext/javaversion.class", "/usr/doc/swig-2.0.6/Examples/r/class"};
			for (String sr : ss) {
				Object []os;
				
				if ((os = trie.getObjects(sr)) != null) {
					System.out.println (sr);
 					for (Object o : os) {
 						System.out.println ("\t" + o);
						
 					}
				}
				
			}
			System.out.println (count);
			
			Runtime runtime = Runtime.getRuntime();

		    NumberFormat format = NumberFormat.getInstance();

		    StringBuilder sb = new StringBuilder();
		    long maxMemory = runtime.maxMemory();
		    long allocatedMemory = runtime.totalMemory();
		    long freeMemory = runtime.freeMemory();

		    sb.append("free memory: " + format.format(freeMemory / 1024) + "<br/>");
		    sb.append("allocated memory: " + format.format(allocatedMemory / 1024) + "<br/>");
		    sb.append("max memory: " + format.format(maxMemory / 1024) + "<br/>");
		    sb.append("total free memory: " + format.format((freeMemory + (maxMemory - allocatedMemory)) / 1024) + "<br/>");
		    System.out.println (sb.toString());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		

	}

}
