package cz.jikos.ales.mff.prgj.cfinder.gui;

/*
 * NPRG013 2012/2013 Project
 * 
 * Ales Novak <alesak@jikos.cz> 
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.TrayIcon.MessageType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.Scanner;
import java.util.Stack;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;
import javax.swing.border.BevelBorder;
import javax.swing.text.MutableAttributeSet;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import cz.jikos.ales.mff.prgj.cfinder.ClsInfo;
import cz.jikos.ales.mff.prgj.cfinder.PkgInfo;
import cz.jikos.ales.mff.prgj.cfinder.Storage;
import cz.jikos.ales.mff.prgj.cfinder.Storage.Callback;
import static javax.swing.GroupLayout.Alignment.*;

/**
 * Main (and only) search window of the application.
 * 
 * @author alesak@jikos.cz
 */
public class ClassFinderFrame extends JFrame implements ActionListener {

	private static final long serialVersionUID = -8734190422356104428L;
	protected Storage storage;
	protected DefaultMutableTreeNode root;
	protected Finder finder;
	protected JTree tree;
	protected JButton btnFind, btnExit;
	protected int found = 0;
	protected JLabel labelFound;

	JLabel statusLabel;

	public void setStorage(Storage storage) {
		this.storage = storage;
	}

	/**
	 * Class managing running query.
	 * 
	 * @author alesak@jikos.cz
	 */
	public class Finder extends SwingWorker<Object, ClsInfo> implements
			Callback {

		protected ClsInfo query;
		protected int limit;

		public Finder(ClsInfo query, int limit, JTree tree) {
			this.query = query;
			this.limit = limit;

		}

		protected DefaultMutableTreeNode nodeForClass(ClsInfo cinfo) {
			String name = "";
			if (cinfo.pkg != null && cinfo.pkg.name.length() > 0) {
				name = cinfo.pkg.name + ".";
			}
			name += cinfo.name;

			DefaultMutableTreeNode pkg, methods, fields, node = new DefaultMutableTreeNode(
					name);
			if (cinfo.fileName != null) {
				node.add(new DefaultMutableTreeNode(cinfo.fileName));
			}

			if (cinfo.methods.size() > 0) {
				methods = new DefaultMutableTreeNode("methods");
				for (ClsInfo.Method m : cinfo.methods) {
					methods.add(new DefaultMutableTreeNode(m.name));
				}
				node.add(methods);
			}
			if (cinfo.fields.size() > 0) {
				fields = new DefaultMutableTreeNode("fields");
				for (ClsInfo.Field f : cinfo.fields)
					fields.add(new DefaultMutableTreeNode(f.name));
				node.add(fields);
			}
			return node;
		}

		@Override
		protected void process(List<ClsInfo> chunks) {
			for (ClsInfo cinfo : chunks) {
				DefaultMutableTreeNode node = nodeForClass(cinfo);
				root.add(node);
			}
			found += chunks.size();
			labelFound.setText (found + " found");

			tree.expandPath(new TreePath(root.getPath()));
			tree.treeDidChange();
			tree.updateUI();
		}

		@Override
		protected void done() {
			super.done();
			btnFind.setEnabled(true);
			tree.updateUI();
			tree.invalidate();
			if (found == 0)
				labelFound.setText("No record found");
			// tree.expandRow(0);
		}

		@Override
		protected Object doInBackground() throws Exception {
			try {
				ClsInfo[] info = storage.findClsInfo(query, limit, this);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return null;
		}

		@Override
		public boolean call(ClsInfo object) {
			publish(object);
			return false;
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnExit)
			this.processWindowEvent(new WindowEvent(this,
					WindowEvent.WINDOW_CLOSING));
	}
	
	/**
	 * Check textfields, if they contain valid query.
	 * @return
	 */
	protected boolean checkValidQuery ()
	{
		JTextField fs[] = {fieldClassName, fieldPackageName, fieldFileName, fieldMethodName};
		
		int l = 0;
		
		for (JTextField f : fs) {
			String t = f.getText();
			l += t.length();
			if (t.contains("*")) {
				if ((! t.endsWith("*") || ((t.length() - t.replace("*", "").length()) > 1))) {
					JOptionPane.showMessageDialog(this, "Star (*) can be only at the end of search phrase", "Bad phrase", JOptionPane.ERROR_MESSAGE);
					return false;
				}
			}
		}
		if (l == 0) {
			JOptionPane.showMessageDialog(this, "Enter at least one search condition", "Bad phrase", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}

	/**
	 * Find pressed, or enter in any textfied typed.
	 */
	protected ActionListener listenerBtn = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			JButton btn = btnFind;//(JButton) e.getSource();
		
			if (btn.isEnabled() == false)
				return;
			
			ClsInfo query = new ClsInfo();

			/* check if at least one textfield is filled
			 * and that no invalid wildcards are typed.
			 */
			if (checkValidQuery () == false) return;
			
			if (fieldClassName.getText().length() > 0)
				query.name = fieldClassName.getText();
			if (fieldPackageName.getText().length() > 0)
				query.pkg = new PkgInfo(fieldPackageName.getText());
			if (fieldFileName.getText().length() > 0)
				query.fileName = fieldFileName.getText();
			if (fieldMethodName.getText().length() > 0) {
				ClsInfo.Method m = new ClsInfo.Method();
				m.name = fieldMethodName.getText();
				query.methods.add(m);
			}
			
			btn.setEnabled(false);
			root.removeAllChildren();
			tree.updateUI();
			tree.treeDidChange();
			found = 0;
			labelFound.setText("");
			
			finder = new Finder(query, 500, null);
			finder.execute();
		}
	};

	protected JTextField fieldClassName;
	protected JTextField fieldPackageName;
	protected JTextField fieldFileName;
	protected JTextField fieldMethodName;

	protected JPanel createSearchPanel() {
		JPanel panel = new JPanel();

		panel.setLayout(new GridLayout(4, 2, 5, 5));
		fieldClassName = new JTextField();
		fieldClassName.addActionListener(listenerBtn);
		panel.add(new JLabel("Class name"));
		panel.add(fieldClassName);

		fieldPackageName = new JTextField();
		fieldPackageName.addActionListener(listenerBtn);
		panel.add(new JLabel("Package name"));
		panel.add(fieldPackageName);

		fieldMethodName = new JTextField();
		fieldMethodName.addActionListener(listenerBtn);
		panel.add(new JLabel("Method name"));
		panel.add(fieldMethodName);

		fieldFileName = new JTextField();
		fieldFileName.addActionListener(listenerBtn);
		panel.add(new JLabel("File name"));
		panel.add(fieldFileName);

		panel.setBorder(BorderFactory.createTitledBorder("Find"));
		return panel;
	}

	protected JPanel createButtonPanel() {
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

		labelFound = new JLabel();
		buttonPanel.add(labelFound);
		
		btnFind = new JButton("Find");
		btnFind.addActionListener(listenerBtn);
		buttonPanel.add(btnFind);

		btnExit = new JButton("Exit");
		btnExit.addActionListener(this);
		buttonPanel.add(btnExit);

		return buttonPanel;
	}

	protected JTree createTree() {
		root = new DefaultMutableTreeNode("Find");
		tree = new JTree(root);
		tree.setRootVisible(false);

		return (tree);
	}

	public ClassFinderFrame() {
		super();

		JPanel panTop = new JPanel();
		panTop.setLayout(new BorderLayout());

		panTop.add(createSearchPanel(), BorderLayout.CENTER);
		panTop.add(createButtonPanel(), BorderLayout.SOUTH);

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(panTop);
		splitPane.setBottomComponent(new JScrollPane(createTree()));
		this.add(splitPane, BorderLayout.CENTER);
		setTitle("ClassFinder");
		pack();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public void work() {

		pack();
		setVisible(true);
	}

}
