package cz.jikos.ales.mff.prgj.simpleser;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.util.Collection;
import java.util.LinkedList;

import cz.jikos.ales.mff.prgj.cfinder.Indexer;

/**
 * This is a simple serializator/deserializator, which uses something
 * as JSON as serializating protocol.
 * 
 * Most types should be further added.
 * 
 * Each type and field which should be serialized should be annotated
 * with SimSer.Serialize.
 * 
 * @author alesak@jikos.cz
 * @param <T> is the type to serialize
 */
public class SimpleSer<T> {
	@SimSer.Serialize
	protected static class TestClass {
		@SimSer.Serialize
		protected String hello;
		
		@SimSer.Serialize
		protected String []hellos;
	
		@SimSer.Serialize
		public int index = -12;
		
		public TestClass () {
			hello = "ahoj ahoj ahoj";
			hellos = new String[]{"Zdar", "Dobry den", "Čum na drát"};
		}
	}
	
	protected String defaultEncoding = "UTF-8";

	
	@SuppressWarnings("unchecked")
	protected void deserializeCollection (InputStream is, Class<?> cls, @SuppressWarnings("rawtypes") Collection c) throws InstantiationException, IllegalAccessException, IOException {
		int num = 0;
		while (true) {
			num ++;
			int b = is.read();
			if (num == 1 && b == '[') {}
			else if (b == ',') {}
			else if (b == ']') break;
			Object o = deserializeObject(is, cls);
			c.add(o);
		}
	}
	
	protected Object deserializeObject (InputStream is, Class<?> c) throws InstantiationException, IllegalAccessException, IOException
	{
		if (c == String.class) {
			return deserializeString (is);
 		} else if (c.isAnnotationPresent(SimSer.Serialize.class)) {
			
			return (deserializeSimObject(is, c));
			
		}
		return (null);
		
	}
	
	@SuppressWarnings("rawtypes")
	protected Object deserializeSimObject (InputStream is, Class<?> c) throws IOException, InstantiationException, IllegalAccessException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		boolean isescape = false;
		int num = 0;
		Object obj = c.newInstance();

		while (true) {
			int r = is.read();
			num ++;
			if (num == 1 && r != '{') throw new RuntimeException("Object not starting with brace!" + r + ":");
			if (isescape) baos.write(r);
			else if (r == '\\') isescape = true;
			else if (r == ',') continue;
			else if (r == '{' && num == 1) continue;
			else if (r == '}') break;
			else if (r == ':') {
				String fieldname = baos.toString(defaultEncoding);
				baos.reset();
				
				try {
					Field f = c.getDeclaredField(fieldname);
					Class<?> fc = f.getType();
					f.setAccessible(true);
					if (fc.isArray()) {
						LinkedList<Object> list = new LinkedList<Object>();
						deserializeCollection (is, fc.getComponentType(), list);
						Object arr = Array.newInstance (fc.getComponentType(), list.size());
						int le = 0;
						for (Object oo : list) {
							Array.set(arr, le ++, oo);
						}
						f.set(obj, arr);

					} else if (f.get(obj) instanceof Collection) {
						deserializeCollection (is, (Class)((ParameterizedType)f.getGenericType()).getActualTypeArguments()[0], (Collection)f.get(obj));
					} else if (fc.isPrimitive()) {
						if (fc == int.class) 
							f.setInt(obj, Integer.valueOf(deserializeString(is)).intValue());
						else if (fc == long.class) 
							f.setLong(obj, Long.valueOf(deserializeString(is)).longValue());
						else {
							throw new RuntimeException ("Unsupported primitive type " + fc.getName());
						}
					} else
						f.set(obj, deserializeObject (is, f.getType()));
				} catch (NoSuchFieldException e) {
					e.printStackTrace();
					throw new RuntimeException("No field " + fieldname + " in class " + c.getName());
				} catch (SecurityException e) {
					e.printStackTrace();
					throw new RuntimeException("No field " + fieldname + " in class " + c.getName());
				} catch (IllegalAccessException e) {
					throw new RuntimeException ("IllegalAccess: " + fieldname);
				}
			}
			else baos.write(r);
		}
		return obj;
	}
	
	
	protected String deserializeString (InputStream is) throws IOException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		boolean isescape = false;
		int num = 0;
		
		while (true) {
			int r = is.read();
			num ++;
			if (isescape) { baos.write(r); isescape = false; }
			else if (r == '\\') isescape = true;
			else if (r == '\"') {
				if (num > 1) break;
			}
			else baos.write(r);
		}
		return baos.toString(defaultEncoding);
	}
	
	protected Collection<Object> getCollection (Object o) {
		LinkedList<Object> ll = new LinkedList <Object> ();
		
		for (int i = 0; i < Array.getLength(o); i ++) {
			ll.add(Array.get(o, i));
		}
		return (ll);
	}
	
	
	@SuppressWarnings("rawtypes")
	protected void serializeCollection (Collection c, OutputStream os) throws IOException, IllegalArgumentException, IllegalAccessException {
		int num = 0;
		
		os.write('[');
		for (Object o : c) {
			if (num > 0) os.write(',');
			serializeObject (o, os);
			num ++;
		}
		os.write(']');
	}

	protected void serializeFieldName (String name, OutputStream os) throws IOException {
		for (byte b : name.getBytes(defaultEncoding)) {
			os.write(b);
		}
		os.write(':');
	}
	
	@SuppressWarnings("rawtypes")
	protected void serializeObject (Object object, OutputStream os) throws IOException, IllegalArgumentException, IllegalAccessException {
		Class<?> c = object.getClass();
		if (c.isAnnotationPresent(SimSer.Serialize.class)) {
			serializeSimObject(c, object, os);
		} else if (object instanceof Collection) {
			
			serializeCollection((Collection)object, os);
		} else if (c.isArray()) {
			serializeCollection(getCollection(object), os);
		} else {
			serializeString(object.toString(), os);
		}
	}
	
	
	@SuppressWarnings("rawtypes")
	protected void serializeSimObject (Class<?> c, Object o, OutputStream os) throws IOException, IllegalArgumentException, IllegalAccessException {
		os.write('{');
		int num = 0;
		//Indexer.log ("serializeSimObject " + c.getName());
		for (Field f: c.getDeclaredFields()) {
			f.setAccessible(true);
			//Indexer.log ("serializeSimObject " + c.getName() + "::" + f.getName());

			Object fo = f.get(o);

			//Indexer.log ("serializeSimObject " + c.getName() + "::object");
			if (! f.isAnnotationPresent(SimSer.Serialize.class)) {continue;}

			if (fo == null) continue;
			if (fo instanceof Collection && ((Collection)fo).size() == 0) continue;
			if (f.getClass().isArray() && Array.getLength(fo) == 0) continue;
			if (num > 0) os.write(',');
			
			serializeFieldName (f.getName(), os);
			serializeObject (fo, os);
			num ++;
		}
		os.write('}');
	}
	
	protected void serializeString (String s, OutputStream os) throws IOException {
		os.write('\"');
		for (byte b : s.getBytes(defaultEncoding)) {
			if (b == '\"') os.write('\\');
			os.write(b);
		}
		os.write('\"');
	}
	
	/**
	 * Serializes object into output stream
	 * @param object object to serialize
	 * @param os outputstream to write.
	 * 
	 * @throws IOException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 */
	public void serialize(T object, OutputStream os) throws IOException, IllegalArgumentException, IllegalAccessException {
		serializeObject (object, os);
		os.flush();
	}

	/**
	 * Reads object of specified type.
	 * 
	 * @param is inputstream to read
	 * @param c class of object to read
	 * @return newly allocated object 
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	public T deserialize(InputStream is, Class<T> c) throws InstantiationException, IllegalAccessException, IOException {
		
		return (T)deserializeObject(is, c);
	}

	static public void main (String[] arg) throws IOException, IllegalArgumentException, IllegalAccessException, InstantiationException {
		TestClass tst = new TestClass();
		SimpleSer<TestClass> tser = new SimpleSer<TestClass>();
		
		tser.serialize(tst, System.out);
	
		ByteArrayInputStream bais = new ByteArrayInputStream("{hello:\"a\",hellos:[\"Tepic\",\"Zdar\",\"Dobry den\",\"Čum na drát\"],index:\"-12\"}".getBytes());
		
		TestClass res = tser.deserialize(bais, TestClass.class);
		
		tser.serialize(res, System.out);
	}

}
